@@ -1,11 +1,24 @@
-from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Sequence, Tuple, cast
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+    cast,
+)
 
 import dagster._check as check
-from dagster import AssetKey, ResourceDefinition
+from dagster import AssetKey
 from dagster._annotations import experimental, public
 from dagster._core.definitions.cacheable_assets import CacheableAssetsDefinition
 from dagster._core.definitions.events import CoercibleToAssetKeyPrefix
 from dagster._core.definitions.freshness_policy import FreshnessPolicy
+from dagster._core.definitions.resource_definition import ResourceDefinition
 from dagster._core.execution.context.init import build_init_resource_context
 from dagster._utils.merger import deep_merge_dicts
 from dagster_managed_elements import (
@@ -112,9 +125,11 @@ def conn_dict(conn: Optional[AirbyteConnection]) -> Mapping[str, Any]:
         "destination": conn.destination.name if conn.destination else "Unknown",
         "normalize data": conn.normalize_data,
         "streams": {k: v.to_json() for k, v in conn.stream_config.items()},
-        "destination namespace": conn.destination_namespace.name
-        if isinstance(conn.destination_namespace, AirbyteDestinationNamespace)
-        else conn.destination_namespace,
+        "destination namespace": (
+            conn.destination_namespace.name
+            if isinstance(conn.destination_namespace, AirbyteDestinationNamespace)
+            else conn.destination_namespace
+        ),
         "prefix": conn.prefix,
     }
 
@@ -634,22 +649,26 @@ class AirbyteManagedElementReconciler(ManagedElementReconciler):
     @public
     def __init__(
         self,
-        airbyte: ResourceDefinition,
+        airbyte: Union[AirbyteResource, ResourceDefinition],
         connections: Iterable[AirbyteConnection],
         delete_unmentioned_resources: bool = False,
     ):
         """Reconciles Python-specified Airbyte connections with an Airbyte instance.
 
         Args:
-            airbyte (ResourceDefinition): The Airbyte resource definition to reconcile against.
+            airbyte (Union[AirbyteResource, ResourceDefinition]): The Airbyte resource definition to reconcile against.
             connections (Iterable[AirbyteConnection]): The Airbyte connection objects to reconcile.
             delete_unmentioned_resources (bool): Whether to delete resources that are not mentioned in
                 the set of connections provided. When True, all Airbyte instance contents are effectively
                 managed by the reconciler. Defaults to False.
         """
-        airbyte = check.inst_param(airbyte, "airbyte", ResourceDefinition)
+        # airbyte = check.inst_param(airbyte, "airbyte", ResourceDefinition)
 
-        self._airbyte_instance: AirbyteResource = airbyte(build_init_resource_context())
+        self._airbyte_instance: AirbyteResource = (
+            airbyte
+            if isinstance(airbyte, AirbyteResource)
+            else airbyte(build_init_resource_context())
+        )
         self._connections = list(
             check.iterable_param(connections, "connections", of_type=AirbyteConnection)
         )
@@ -681,7 +700,7 @@ class AirbyteManagedElementReconciler(ManagedElementReconciler):
 class AirbyteManagedElementCacheableAssetsDefinition(AirbyteInstanceCacheableAssetsDefinition):
     def __init__(
         self,
-        airbyte_resource_def: ResourceDefinition,
+        airbyte_resource_def: AirbyteResource,
         key_prefix: Sequence[str],
         create_assets_for_normalization_tables: bool,
         connection_to_group_fn: Optional[Callable[[str], Optional[str]]],
@@ -715,14 +734,14 @@ class AirbyteManagedElementCacheableAssetsDefinition(AirbyteInstanceCacheableAss
                 )
             )
         elif isinstance(diff, ManagedElementError):
-            raise ValueError(f"Error checking Airbyte connections: {str(diff)}")
+            raise ValueError(f"Error checking Airbyte connections: {diff}")
 
         return super()._get_connections()
 
 
 @experimental
 def load_assets_from_connections(
-    airbyte: ResourceDefinition,
+    airbyte: Union[AirbyteResource, ResourceDefinition],
     connections: Iterable[AirbyteConnection],
     key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
     create_assets_for_normalization_tables: bool = True,
@@ -740,7 +759,7 @@ def load_assets_from_connections(
     This method will raise an error on repo load if the passed AirbyteConnection objects are not in sync with the Airbyte instance.
 
     Args:
-        airbyte (ResourceDefinition): An AirbyteResource configured with the appropriate connection
+        airbyte (Union[AirbyteResource, ResourceDefinition]): An AirbyteResource configured with the appropriate connection
             details.
         connections (Iterable[AirbyteConnection]): A list of AirbyteConnection objects to build assets for.
         key_prefix (Optional[CoercibleToAssetKeyPrefix]): A prefix for the asset keys created.
@@ -767,15 +786,13 @@ def load_assets_from_connections(
 
         from dagster_airbyte import (
             AirbyteConnection,
-            airbyte_resource,
+            AirbyteResource,
             load_assets_from_connections,
         )
 
-        airbyte_instance = airbyte_resource.configured(
-            {
-                "host": "localhost",
-                "port": "8000",
-            }
+        airbyte_instance = AirbyteResource(
+                host: "localhost",
+                port: "8000",
         )
         airbyte_connections = [
             AirbyteConnection(...),
@@ -795,7 +812,11 @@ def load_assets_from_connections(
         connection_to_io_manager_key_fn = lambda _: io_manager_key
 
     return AirbyteManagedElementCacheableAssetsDefinition(
-        airbyte_resource_def=check.inst_param(airbyte, "airbyte", ResourceDefinition),
+        airbyte_resource_def=(
+            airbyte
+            if isinstance(airbyte, AirbyteResource)
+            else airbyte(build_init_resource_context())
+        ),
         key_prefix=key_prefix,
         create_assets_for_normalization_tables=check.bool_param(
             create_assets_for_normalization_tables, "create_assets_for_normalization_tables"
