@@ -3,20 +3,23 @@ import json
 import logging
 import sys
 import time
+from abc import abstractmethod
 from contextlib import contextmanager
 from typing import Any, Dict, List, Mapping, Optional, cast
 
 import requests
 from dagster import (
+    ConfigurableResource,
     Failure,
-    Field,
-    StringSource,
     _check as check,
     get_dagster_logger,
     resource,
 )
-from dagster._config.field_utils import Permissive
+from dagster._config.pythonic_config import infer_schema_from_config_class
+from dagster._core.definitions.resource_definition import dagster_maintained_resource
+from dagster._utils.cached_method import cached_method
 from dagster._utils.merger import deep_merge_dicts
+from pydantic import Field
 from requests.exceptions import RequestException
 
 from dagster_airbyte.types import AirbyteOutput
@@ -34,70 +37,367 @@ class AirbyteState:
     INCOMPLETE = "incomplete"
 
 
-class AirbyteResource:
-    """This class exposes methods on top of the Airbyte REST API."""
-
-    def __init__(
-        self,
-        host: str,
-        port: str,
-        use_https: bool,
-        request_max_retries: int = 3,
-        request_retry_delay: float = 0.25,
-        request_timeout: int = 15,
-        request_additional_params: Optional[Mapping[str, Any]] = None,
-        log: logging.Logger = get_dagster_logger(),
-        forward_logs: bool = True,
-        cancel_sync_on_run_termination: bool = True,
-        username: Optional[str] = None,
-        password: Optional[str] = None,
-    ):
-        self._host = host
-        self._port = port
-        self._use_https = use_https
-        self._request_max_retries = request_max_retries
-        self._request_retry_delay = request_retry_delay
-        self._request_timeout = request_timeout
-        self._additional_request_params = request_additional_params or dict()
-
-        self._log = log
-
-        self._forward_logs = forward_logs
-        self._request_cache: Dict[str, Optional[Mapping[str, object]]] = {}
+class AirbyteResourceState:
+    def __init__(self) -> None:
+        self.request_cache: Dict[str, Optional[Mapping[str, object]]] = {}
         # Int in case we nest contexts
-        self._cache_enabled = 0
+        self.cache_enabled = 0
 
-        self._username = username
-        self._password = password
 
-        self._cancel_sync_on_run_termination = cancel_sync_on_run_termination
+class BaseAirbyteResource(ConfigurableResource):
+    request_max_retries: int = Field(
+        default=3,
+        description=(
+            "The maximum number of times requests to the Airbyte API should be retried "
+            "before failing."
+        ),
+    )
+    request_retry_delay: float = Field(
+        default=0.25,
+        description="Time (in seconds) to wait between each request retry.",
+    )
+    request_timeout: int = Field(
+        default=15,
+        description="Time (in seconds) after which the requests to Airbyte are declared timed out.",
+    )
+    cancel_sync_on_run_termination: bool = Field(
+        default=True,
+        description=(
+            "Whether to cancel a sync in Airbyte if the Dagster runner is terminated. This may"
+            " be useful to disable if using Airbyte sources that cannot be cancelled and"
+            " resumed easily, or if your Dagster deployment may experience runner interruptions"
+            " that do not impact your Airbyte deployment."
+        ),
+    )
+    poll_interval: float = Field(
+        default=DEFAULT_POLL_INTERVAL_SECONDS,
+        description="Time (in seconds) to wait between checking a sync's status.",
+    )
+
+    @classmethod
+    def _is_dagster_maintained(cls) -> bool:
+        return True
+
+    @property
+    @cached_method
+    def _log(self) -> logging.Logger:
+        return get_dagster_logger()
+
+    @property
+    @abstractmethod
+    def api_base_url(self) -> str:
+        raise NotImplementedError()
+
+    @property
+    @abstractmethod
+    def all_additional_request_params(self) -> Mapping[str, Any]:
+        raise NotImplementedError()
+
+    def make_request(
+        self, endpoint: str, data: Optional[Mapping[str, object]] = None, method: str = "POST"
+    ) -> Optional[Mapping[str, object]]:
+        """Creates and sends a request to the desired Airbyte REST API endpoint.
+
+        Args:
+            endpoint (str): The Airbyte API endpoint to send this request to.
+            data (Optional[str]): JSON-formatted data string to be included in the request.
+
+        Returns:
+            Optional[Dict[str, Any]]: Parsed json data from the response to this request
+        """
+        url = self.api_base_url + endpoint
+        headers = {"accept": "application/json"}
+
+        num_retries = 0
+        while True:
+            try:
+                request_args: Dict[str, Any] = dict(
+                    method=method,
+                    url=url,
+                    headers=headers,
+                    timeout=self.request_timeout,
+                )
+                if data:
+                    request_args["json"] = data
+
+                request_args = deep_merge_dicts(
+                    request_args,
+                    self.all_additional_request_params,
+                )
+
+                response = requests.request(
+                    **request_args,
+                )
+                response.raise_for_status()
+                if response.status_code == 204:
+                    return None
+                return response.json()
+            except RequestException as e:
+                self._log.error("Request to Airbyte API failed: %s", e)
+                if num_retries == self.request_max_retries:
+                    break
+                num_retries += 1
+                time.sleep(self.request_retry_delay)
+
+        raise Failure(f"Max retries ({self.request_max_retries}) exceeded with url: {url}.")
+
+    @abstractmethod
+    def start_sync(self, connection_id: str) -> Mapping[str, object]:
+        raise NotImplementedError()
+
+    @abstractmethod
+    def get_connection_details(self, connection_id: str) -> Mapping[str, object]:
+        raise NotImplementedError()
+
+    @abstractmethod
+    def get_job_status(self, connection_id: str, job_id: int) -> Mapping[str, object]:
+        raise NotImplementedError()
+
+    @abstractmethod
+    def cancel_job(self, job_id: int):
+        raise NotImplementedError()
+
+    @property
+    @abstractmethod
+    def _should_forward_logs(self) -> bool:
+        raise NotImplementedError()
+
+    def sync_and_poll(
+        self,
+        connection_id: str,
+        poll_interval: Optional[float] = None,
+        poll_timeout: Optional[float] = None,
+    ) -> AirbyteOutput:
+        """Initializes a sync operation for the given connector, and polls until it completes.
+
+        Args:
+            connection_id (str): The Airbyte Connector ID. You can retrieve this value from the
+                "Connection" tab of a given connection in the Arbyte UI.
+            poll_interval (float): The time (in seconds) that will be waited between successive polls.
+            poll_timeout (float): The maximum time that will waited before this operation is timed
+                out. By default, this will never time out.
+
+        Returns:
+            :py:class:`~AirbyteOutput`:
+                Details of the sync job.
+        """
+        connection_details = self.get_connection_details(connection_id)
+        job_details = self.start_sync(connection_id)
+        job_info = cast(Dict[str, object], job_details.get("job", {}))
+        job_id = cast(int, job_info.get("id"))
+
+        self._log.info(f"Job {job_id} initialized for connection_id={connection_id}.")
+        start = time.monotonic()
+        logged_attempts = 0
+        logged_lines = 0
+        state = None
+
+        try:
+            while True:
+                if poll_timeout and start + poll_timeout < time.monotonic():
+                    raise Failure(
+                        f"Timeout: Airbyte job {job_id} is not ready after the timeout"
+                        f" {poll_timeout} seconds"
+                    )
+                time.sleep(poll_interval or self.poll_interval)
+                job_details = self.get_job_status(connection_id, job_id)
+                attempts = cast(List, job_details.get("attempts", []))
+                cur_attempt = len(attempts)
+                # spit out the available Airbyte log info
+                if cur_attempt:
+                    if self._should_forward_logs:
+                        log_lines = attempts[logged_attempts].get("logs", {}).get("logLines", [])
+
+                        for line in log_lines[logged_lines:]:
+                            sys.stdout.write(line + "\n")
+                            sys.stdout.flush()
+                        logged_lines = len(log_lines)
+
+                    # if there's a next attempt, this one will have no more log messages
+                    if logged_attempts < cur_attempt - 1:
+                        logged_lines = 0
+                        logged_attempts += 1
+
+                job_info = cast(Dict[str, object], job_details.get("job", {}))
+                state = job_info.get("status")
+
+                if state in (AirbyteState.RUNNING, AirbyteState.PENDING, AirbyteState.INCOMPLETE):
+                    continue
+                elif state == AirbyteState.SUCCEEDED:
+                    break
+                elif state == AirbyteState.ERROR:
+                    raise Failure(f"Job failed: {job_id}")
+                elif state == AirbyteState.CANCELLED:
+                    raise Failure(f"Job was cancelled: {job_id}")
+                else:
+                    raise Failure(f"Encountered unexpected state `{state}` for job_id {job_id}")
+        finally:
+            # if Airbyte sync has not completed, make sure to cancel it so that it doesn't outlive
+            # the python process
+            if (
+                state not in (AirbyteState.SUCCEEDED, AirbyteState.ERROR, AirbyteState.CANCELLED)
+                and self.cancel_sync_on_run_termination
+            ):
+                self.cancel_job(job_id)
+
+        return AirbyteOutput(job_details=job_details, connection_details=connection_details)
+
+
+class AirbyteCloudResource(BaseAirbyteResource):
+    """This resource allows users to programatically interface with the Airbyte Cloud API to launch
+    syncs and monitor their progress.
+
+    **Examples:**
+
+    .. code-block:: python
+
+        from dagster import job, EnvVar
+        from dagster_airbyte import AirbyteResource
+
+        my_airbyte_resource = AirbyteCloudResource(
+            api_key=EnvVar("AIRBYTE_API_KEY"),
+        )
+
+        airbyte_assets = build_airbyte_assets(
+            connection_id="87b7fe85-a22c-420e-8d74-b30e7ede77df",
+            destination_tables=["releases", "tags", "teams"],
+        )
+
+        defs = Definitions(
+            assets=[airbyte_assets],
+            resources={"airbyte": my_airbyte_resource},
+        )
+    """
+
+    api_key: str = Field(..., description="The Airbyte Cloud API key.")
+
+    @property
+    def api_base_url(self) -> str:
+        return "https://api.airbyte.com/v1"
+
+    @property
+    def all_additional_request_params(self) -> Mapping[str, Any]:
+        return {"headers": {"Authorization": f"Bearer {self.api_key}", "User-Agent": "dagster"}}
+
+    def start_sync(self, connection_id: str) -> Mapping[str, object]:
+        job_sync = check.not_none(
+            self.make_request(
+                endpoint="/jobs",
+                data={
+                    "connectionId": connection_id,
+                    "jobType": "sync",
+                },
+            )
+        )
+        return {"job": {"id": job_sync["jobId"], "status": job_sync["status"]}}
+
+    def get_connection_details(self, connection_id: str) -> Mapping[str, object]:
+        return {}
+
+    def get_job_status(self, connection_id: str, job_id: int) -> Mapping[str, object]:
+        job_status = check.not_none(self.make_request(endpoint=f"/jobs/{job_id}", method="GET"))
+        return {"job": {"id": job_status["jobId"], "status": job_status["status"]}}
+
+    def cancel_job(self, job_id: int):
+        self.make_request(endpoint=f"/jobs/{job_id}", method="DELETE")
+
+    @property
+    def _should_forward_logs(self) -> bool:
+        # Airbyte Cloud does not support streaming logs yet
+        return False
+
+
+class AirbyteResource(BaseAirbyteResource):
+    """This resource allows users to programatically interface with the Airbyte REST API to launch
+    syncs and monitor their progress.
+
+    **Examples:**
+
+    .. code-block:: python
+
+        from dagster import job, EnvVar
+        from dagster_airbyte import AirbyteResource
+
+        my_airbyte_resource = AirbyteResource(
+            host=EnvVar("AIRBYTE_HOST"),
+            port=EnvVar("AIRBYTE_PORT"),
+            # If using basic auth
+            username=EnvVar("AIRBYTE_USERNAME"),
+            password=EnvVar("AIRBYTE_PASSWORD"),
+        )
+
+        airbyte_assets = build_airbyte_assets(
+            connection_id="87b7fe85-a22c-420e-8d74-b30e7ede77df",
+            destination_tables=["releases", "tags", "teams"],
+        )
+
+        defs = Definitions(
+            assets=[airbyte_assets],
+            resources={"airbyte": my_airbyte_resource},
+        )
+    """
+
+    host: str = Field(description="The Airbyte server address.")
+    port: str = Field(description="Port used for the Airbyte server.")
+    username: Optional[str] = Field(default=None, description="Username if using basic auth.")
+    password: Optional[str] = Field(default=None, description="Password if using basic auth.")
+    use_https: bool = Field(
+        default=False, description="Whether to use HTTPS to connect to the Airbyte server."
+    )
+    forward_logs: bool = Field(
+        default=True,
+        description=(
+            "Whether to forward Airbyte logs to the compute log, can be expensive for"
+            " long-running syncs."
+        ),
+    )
+    request_additional_params: Mapping[str, Any] = Field(
+        default=dict(),
+        description=(
+            "Any additional kwargs to pass to the requests library when making requests to Airbyte."
+        ),
+    )
+
+    @property
+    @cached_method
+    def _state(self) -> AirbyteResourceState:
+        return AirbyteResourceState()
+
+    @property
+    @cached_method
+    def _log(self) -> logging.Logger:
+        return get_dagster_logger()
 
     @property
     def api_base_url(self) -> str:
         return (
-            ("https://" if self._use_https else "http://")
-            + (f"{self._host}:{self._port}" if self._port else self._host)
+            ("https://" if self.use_https else "http://")
+            + (f"{self.host}:{self.port}" if self.port else self.host)
             + "/api/v1"
         )
 
+    @property
+    def _should_forward_logs(self) -> bool:
+        return self.forward_logs
+
     @contextmanager
     def cache_requests(self):
         """Context manager that enables caching certain requests to the Airbyte API,
         cleared when the context is exited.
         """
         self.clear_request_cache()
-        self._cache_enabled += 1
+        self._state.cache_enabled += 1
         try:
             yield
         finally:
             self.clear_request_cache()
-            self._cache_enabled -= 1
+            self._state.cache_enabled -= 1
 
-    def clear_request_cache(self):
-        self._request_cache = {}
+    def clear_request_cache(self) -> None:
+        self._state.request_cache = {}
 
     def make_request_cached(self, endpoint: str, data: Optional[Mapping[str, object]]):
-        if not self._cache_enabled > 0:
+        if not self._state.cache_enabled > 0:
             return self.make_request(endpoint, data)
         data_json = json.dumps(data, sort_keys=True)
         sha = hashlib.sha1()
@@ -105,9 +405,16 @@ class AirbyteResource:
         sha.update(data_json.encode("utf-8"))
         digest = sha.hexdigest()
 
-        if digest not in self._request_cache:
-            self._request_cache[digest] = self.make_request(endpoint, data)
-        return self._request_cache[digest]
+        if digest not in self._state.request_cache:
+            self._state.request_cache[digest] = self.make_request(endpoint, data)
+        return self._state.request_cache[digest]
+
+    @property
+    def all_additional_request_params(self) -> Mapping[str, Any]:
+        auth_param = (
+            {"auth": (self.username, self.password)} if self.username and self.password else {}
+        )
+        return {**auth_param, **self.request_additional_params}
 
     def make_request(
         self, endpoint: str, data: Optional[Mapping[str, object]]
@@ -134,12 +441,14 @@ class AirbyteResource:
                             url=url,
                             headers=headers,
                             json=data,
-                            timeout=self._request_timeout,
-                            auth=(self._username, self._password)
-                            if self._username and self._password
-                            else None,
+                            timeout=self.request_timeout,
+                            auth=(
+                                (self.username, self.password)
+                                if self.username and self.password
+                                else None
+                            ),
                         ),
-                        self._additional_request_params,
+                        self.request_additional_params,
                     ),
                 )
                 response.raise_for_status()
@@ -148,12 +457,12 @@ class AirbyteResource:
                 return response.json()
             except RequestException as e:
                 self._log.error("Request to Airbyte API failed: %s", e)
-                if num_retries == self._request_max_retries:
+                if num_retries == self.request_max_retries:
                     break
                 num_retries += 1
-                time.sleep(self._request_retry_delay)
+                time.sleep(self.request_retry_delay)
 
-        raise Failure(f"Max retries ({self._request_max_retries}) exceeded with url: {url}.")
+        raise Failure(f"Max retries ({self.request_max_retries}) exceeded with url: {url}.")
 
     def cancel_job(self, job_id: int):
         self.make_request(endpoint="/jobs/cancel", data={"id": job_id})
@@ -216,8 +525,10 @@ class AirbyteResource:
 
     def does_dest_support_normalization(
         self, destination_definition_id: str, workspace_id: str
-    ) -> Dict[str, Any]:
-        return cast(
+    ) -> bool:
+        # Airbyte API changed source of truth for normalization in PR
+        # https://github.com/airbytehq/airbyte/pull/21005
+        norm_dest_def_spec: bool = cast(
             Dict[str, Any],
             check.not_none(
                 self.make_request_cached(
@@ -230,8 +541,26 @@ class AirbyteResource:
             ),
         ).get("supportsNormalization", False)
 
+        norm_dest_def: bool = (
+            cast(
+                Dict[str, Any],
+                check.not_none(
+                    self.make_request_cached(
+                        endpoint="/destination_definitions/get",
+                        data={
+                            "destinationDefinitionId": destination_definition_id,
+                        },
+                    )
+                ),
+            )
+            .get("normalizationConfig", {})
+            .get("supported", False)
+        )
+
+        return any([norm_dest_def_spec, norm_dest_def])
+
     def get_job_status(self, connection_id: str, job_id: int) -> Mapping[str, object]:
-        if self._forward_logs:
+        if self.forward_logs:
             return check.not_none(self.make_request(endpoint="/jobs/get", data={"id": job_id}))
         else:
             # the "list all jobs" endpoint doesn't return logs, which actually makes it much more
@@ -264,7 +593,7 @@ class AirbyteResource:
     def sync_and_poll(
         self,
         connection_id: str,
-        poll_interval: float = DEFAULT_POLL_INTERVAL_SECONDS,
+        poll_interval: Optional[float] = None,
         poll_timeout: Optional[float] = None,
     ) -> AirbyteOutput:
         """Initializes a sync operation for the given connector, and polls until it completes.
@@ -298,13 +627,13 @@ class AirbyteResource:
                         f"Timeout: Airbyte job {job_id} is not ready after the timeout"
                         f" {poll_timeout} seconds"
                     )
-                time.sleep(poll_interval)
+                time.sleep(poll_interval or self.poll_interval)
                 job_details = self.get_job_status(connection_id, job_id)
                 attempts = cast(List, job_details.get("attempts", []))
                 cur_attempt = len(attempts)
                 # spit out the available Airbyte log info
                 if cur_attempt:
-                    if self._forward_logs:
+                    if self.forward_logs:
                         log_lines = attempts[logged_attempts].get("logs", {}).get("logLines", [])
 
                         for line in log_lines[logged_lines:]:
@@ -335,88 +664,15 @@ class AirbyteResource:
             # the python process
             if (
                 state not in (AirbyteState.SUCCEEDED, AirbyteState.ERROR, AirbyteState.CANCELLED)
-                and self._cancel_sync_on_run_termination
+                and self.cancel_sync_on_run_termination
             ):
                 self.cancel_job(job_id)
 
         return AirbyteOutput(job_details=job_details, connection_details=connection_details)
 
 
-@resource(
-    config_schema={
-        "host": Field(
-            StringSource,
-            is_required=True,
-            description="The Airbyte Server Address.",
-        ),
-        "port": Field(
-            StringSource,
-            is_required=True,
-            description="Port for the Airbyte Server.",
-        ),
-        "username": Field(
-            StringSource,
-            description="Username if using basic auth.",
-            is_required=False,
-        ),
-        "password": Field(
-            StringSource,
-            description="Password if using basic auth.",
-            is_required=False,
-        ),
-        "use_https": Field(
-            bool,
-            default_value=False,
-            description="Use https to connect in Airbyte Server.",
-        ),
-        "request_max_retries": Field(
-            int,
-            default_value=3,
-            description=(
-                "The maximum number of times requests to the Airbyte API should be retried "
-                "before failing."
-            ),
-        ),
-        "request_retry_delay": Field(
-            float,
-            default_value=0.25,
-            description="Time (in seconds) to wait between each request retry.",
-        ),
-        "request_timeout": Field(
-            int,
-            default_value=15,
-            description=(
-                "Time (in seconds) after which the requests to Airbyte are declared timed out."
-            ),
-        ),
-        "request_additional_params": Field(
-            Permissive(),
-            description=(
-                "Any additional kwargs to pass to the requests library when making requests to"
-                " Airbyte."
-            ),
-        ),
-        "forward_logs": Field(
-            bool,
-            default_value=True,
-            description=(
-                "Whether to forward Airbyte logs to the compute log, can be expensive for"
-                " long-running syncs."
-            ),
-        ),
-        "cancel_sync_on_run_termination": Field(
-            bool,
-            default_value=True,
-            description=(
-                "Whether to cancel a sync in Airbyte if the Dagster runner is terminated. This may"
-                " be useful to disable if using Airbyte sources that cannot be cancelled and"
-                " resumed easily, or if your Dagster deployment may experience runner interruptions"
-                " that do not impact your Airbyte deployment."
-            ),
-        ),
-    },
-    description="This resource helps manage Airbyte connectors",
-)
+@dagster_maintained_resource
+@resource(config_schema=AirbyteResource.to_config_schema())
 def airbyte_resource(context) -> AirbyteResource:
     """This resource allows users to programatically interface with the Airbyte REST API to launch
     syncs and monitor their progress. This currently implements only a subset of the functionality
@@ -450,17 +706,15 @@ def airbyte_resource(context) -> AirbyteResource:
             ...
 
     """
-    return AirbyteResource(
-        host=context.resource_config["host"],
-        port=context.resource_config["port"],
-        use_https=context.resource_config["use_https"],
-        request_max_retries=context.resource_config["request_max_retries"],
-        request_retry_delay=context.resource_config["request_retry_delay"],
-        request_timeout=context.resource_config["request_timeout"],
-        request_additional_params=context.resource_config["request_additional_params"],
-        log=context.log,
-        forward_logs=context.resource_config["forward_logs"],
-        cancel_sync_on_run_termination=context.resource_config["cancel_sync_on_run_termination"],
-        username=context.resource_config.get("username"),
-        password=context.resource_config.get("password"),
-    )
+    return AirbyteResource.from_resource_context(context)
+
+
+@dagster_maintained_resource
+@resource(config_schema=infer_schema_from_config_class(AirbyteCloudResource))
+def airbyte_cloud_resource(context) -> AirbyteCloudResource:
+    """This resource allows users to programatically interface with the Airbyte Cloud REST API to launch
+    syncs and monitor their progress. Currently, this resource may only be used with the more basic
+    `dagster-airbyte` APIs, including the ops and assets.
+
+    """
+    return AirbyteCloudResource.from_resource_context(context)
